@@ -2,6 +2,7 @@
 # John Fox, Sanford Weisberg, and Jangman Hong
 #  last modified 2012-12-08 by J. Fox
 #  10/31/2012 modifed effect.lm to use z distn for ses with mer and nlme objects
+# 12-21-2012 Allow for empty cells in factor interactions, S. Weisberg
 
 effect <- function(term, mod, ...){
 	UseMethod("effect", mod)
@@ -114,11 +115,33 @@ effect.lm <- function (term, mod, xlevels = list(), default.levels = 10, given.v
         wts <- rep(1, length(mod$residuals))
     mod.matrix <- fixup.model.matrix(mod, mod.matrix, model.matrix(mod), 
                                      X.mod, mod.aug, factor.cols, cnames, term, typical, given.values)
-    effect <- off + mod.matrix %*% mod$coefficients
+# look for aliased coefficients and remove those columns from mod.matrix
+  mod.matrix <- mod.matrix[, !is.na(mod$coefficients)]
+	effect <- off + mod.matrix %*% mod$coefficients[!is.na(mod$coefficients)]
+# end
     result <- list(term = term, formula = formula(mod), response = response.name(mod), 
                    variables = x, fit = effect, x = predict.data[, 1:n.basic, 
                                                                  drop = FALSE], model.matrix = mod.matrix, data = X, 
                    discrepancy = 0, offset=off)
+# find empty cells, if any, and correct
+  whichFact <- unlist(lapply(result$variables, function(x) x$is.factor))
+  zeroes <- NULL
+   if(sum(whichFact) > 1){
+       nameFact <- names(whichFact)[whichFact]
+       counts <- xtabs(as.formula( paste("~", paste(nameFact, collapse="+"))), 
+                   model.frame(mod))
+       zeroes <- which(counts == 0)  
+       }
+  if(length(zeroes) > 0){
+       levs <- expand.grid(lapply(result$variables, function(x) x$levels)) 
+       good <- rep(TRUE, dim(levs)[1])
+       for(z in zeroes){
+           good <- good &  
+                    apply(levs, 1, function(x) !all(x == levs[z, whichFact]))
+       } 
+       result$fit[!good] <- NA
+  } 
+# end of change
     if (se) {
         if (any(family(mod)$family == c("binomial", "poisson")) |
             inherits(mod, "fakeglm")) {
@@ -142,6 +165,13 @@ effect.lm <- function (term, mod, xlevels = list(), default.levels = 10, given.v
         result$lower <- effect - z * result$se
         result$upper <- effect + z * result$se
         result$confidence.level <- confidence.level
+# zero cells
+		if(length(zeroes) > 0){
+		   result$se[!good] <- NA
+		   result$lower[!good] <- NA
+		   result$upper[!good] <- NA
+    }
+# end zero cells
     }
     if (is.null(transformation$link) && is.null(transformation$inverse)) {
         transformation$link <- I
@@ -524,6 +554,32 @@ effect.multinom <- function(term, mod,
                                      lower.logit=Lower.logit, upper.logit=Upper.logit, 
                                      lower.prob=Lower.P, upper.prob=Upper.P,
                                      confidence.level=confidence.level))
+# find empty cells, if any, and correct
+  whichFact <- unlist(lapply(result$variables, function(x) x$is.factor))
+  zeroes <- NULL
+   if(sum(whichFact) > 1){
+       nameFact <- names(whichFact)[whichFact]
+       counts <- xtabs(as.formula( paste("~", paste(nameFact, collapse="+"))), 
+                   model.frame(mod))
+       zeroes <- which(counts == 0)  
+       }
+  if(length(zeroes) > 0){
+       levs <- expand.grid(lapply(result$variables, function(x) x$levels)) 
+       good <- rep(TRUE, dim(levs)[1])
+       for(z in zeroes){
+           good <- good &  
+                    apply(levs, 1, function(x) !all(x == levs[z, whichFact]))
+       }
+       result$prob[!good, ] <- NA
+       result$logit[!good, ] <- NA 
+       if (se){
+           result$se.prob[!good, ] <- NA
+           result$se.logit[!good, ] <- NA
+           result$lower.prob[!good, ] <- NA
+           result$upper.prob[!good, ] <- NA
+           }
+  } 
+# end of change
     class(result) <-'effpoly'
     result
 }
