@@ -3,6 +3,12 @@
 #  last modified 2012-11-30 by J. Fox
 #  29 June 2011 added grid, rotx and roty arguments to the two plot methods
 #   by S. Weisberg
+#  21 Dec 2012 modest modification of empty cells with crossed factors
+#  2013-01-17: Added factor.ci.style arg to plot.eff() and plot.effpoly(). J. Fox
+#  2013-01-18: Added CI bars to multiline plots with factor.ci.style="bars"
+#  2013-01-19: Renamed 'factor.ci.style' to 'ci.style'.  Added a 'none' option
+#   extended to variate terms if multiline=TRUE, ci.style="bars"
+#  2013-01-30: scale arrow "heads" for error bars relative to cex
 
 
 summary.eff <- function(object, type=c("response", "link"), ...){
@@ -113,7 +119,7 @@ make.ticks <- function(range, link, inverse, at, n) {
 }
 
 range.adj <- function(x){
-    range <- range(x)
+    range <- range(x, na.rm=TRUE)
     c(range[1] - .025*(range[2] - range[1]),                                              
       range[2] + .025*(range[2] - range[1]))
 }
@@ -124,31 +130,22 @@ plot.eff <- function(x, x.var=which.max(levels),
     z.var=which.min(levels), multiline=is.null(x$se), rug=TRUE, xlab,
     ylab, main=paste(effect, "effect plot"),
     colors=palette(), symbols=1:10, lines=1:10, cex=1.5, ylim, xlim=NULL,
-    factor.names=TRUE, type=c("response", "link"), ticks=list(at=NULL, n=5),  
+    factor.names=TRUE, ci.style, 
+    type=c("response", "link"), ticks=list(at=NULL, n=5),  
     alternating=TRUE, rotx=0, roty=0, grid=FALSE, layout, rescale.axis=TRUE, 
     transform.x=NULL, ticks.x=NULL,
     key.args=NULL, 
-    row=1, col=1, nrow=1, ncol=1, more=FALSE, ...){
-#     make.ticks <- function(range, link, inverse, at, n) {
-#         link <- if (is.null(link)) 
-#             function(x) nlm(function(y) (inverse(y) - x)^2, 
-#                 mean(range))$estimate
-#         else link
-#         if (is.null(n)) n <- 5
-#         labels <- if (is.null(at)){
-#             labels <- pretty(sapply(range, inverse), n=n+1)
-#         }
-#         else at
-#         ticks <- sapply(labels, link)
-#         list(at=ticks, labels=format(labels))
-#     }
+    row=1, col=1, nrow=1, ncol=1, more=FALSE, ...)
+{  
+    ci.style <- if(missing(ci.style)) NULL else 
+        match.arg(ci.style, c("bars", "lines", "none")) 
     type <- match.arg(type)
     thresholds <- x$thresholds
     has.thresholds <- !is.null(thresholds)
     if (missing(ylab)){
         ylab <- if (has.thresholds) paste(x$response, ": ", paste(x$y.levels, collapse=", "), sep="")
         else x$response
-    }
+    }     
     if (has.thresholds){ 
         threshold.labels <- abbreviate(x$y.levels, minlength=1)
         threshold.labels <- paste(" ", 
@@ -158,9 +155,9 @@ plot.eff <- function(x, x.var=which.max(levels),
     trans.link <- x$transformation$link
     trans.inverse <- x$transformation$inverse
     if (!rescale.axis){
-        x$lower <- trans.inverse(x$lower)
-        x$upper <- trans.inverse(x$upper)
-        x$fit <- trans.inverse(x$fit)
+        x$lower[!is.na(x$lower)] <- trans.inverse(x$lower[!is.na(x$lower)])
+        x$upper[!is.na(x$upper)] <- trans.inverse(x$upper[!is.na(x$upper)])
+        x$fit[!is.na(x$fit)] <- trans.inverse(x$fit)[!is.na(x$fit)]
         trans.link <- trans.inverse <- I
     }
     require(lattice)
@@ -177,23 +174,34 @@ plot.eff <- function(x, x.var=which.max(levels),
     has.se <- !is.null(x$se)
     n.predictors <- ncol(x) - 1 - 3*has.se
     if (n.predictors == 1){
-        range <- if (has.se) range(c(x$lower, x$upper)) else range(x$fit)
-        ylim <- if (!missing(ylim)) ylim else c(range[1] - .025*(range[2] - range[1]),                                              
-            range[2] + .025*(range[2] - range[1]))
-        tickmarks <- if (type == "response") make.ticks(ylim, link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
-        else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
-        if (is.factor(x[,1])){
+### factor no other predictors
+       if (is.factor(x[,1])){
+            ci.style <- if(is.null(ci.style)) "bars" else ci.style
+            range <- if(has.se & ci.style!="none")
+                 range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
+            ylim <- if (!missing(ylim)) ylim else c(range[1] - .025*(range[2] - range[1]),                                              
+                             range[2] + .025*(range[2] - range[1]))
+            tickmarks <- if (type == "response") make.ticks(ylim, 
+                         link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
+                         else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
             levs <- levels(x[,1])  
             plot <- xyplot(eval(parse(
                 text=paste("fit ~ as.numeric(", names(x)[1], ")"))), 
                 strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
                 panel=function(x, y, lower, upper, has.se, ...){
                     if (grid) panel.grid()
-                    llines(x, y, lwd=2, col=colors[1], type='b', pch=19, cex=cex, ...)
-                    if (has.se){
-                        llines(x, lower, lty=2, col=colors[2])
-                        llines(x, upper, lty=2, col=colors[2])
+                    good <- !is.na(y)
+                    if (has.se){ 
+                        if (ci.style == "bars"){
+                            larrows(x0=x[good], y0=lower[good], x1=x[good], y1=upper[good], angle=90, 
+                                code=3, col=colors[2], length=0.125*cex/1.5)
+                        }
+                        else{ if(ci.style == "lines") {
+                            llines(x[good], lower[good], lty=2, col=colors[2])
+                            llines(x[good], upper[good], lty=2, col=colors[2])
+                        }}
                     }
+                    llines(x[good], y[good], lwd=2, col=colors[1], type='b', pch=19, cex=cex, ...)
                     if (has.thresholds){
                         panel.abline(h=thresholds, lty=3)
                         panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)), 
@@ -215,8 +223,17 @@ plot.eff <- function(x, x.var=which.max(levels),
             result$split <- split
             result$more <- more
             class(result) <- c("plot.eff", class(result))
-        }        
+        }  
+### variate, no other predictors      
         else {
+            ci.style <- if(is.null(ci.style)) "lines" else ci.style
+            range <- if(has.se & ci.style!="none")
+                 range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
+                    ylim <- if (!missing(ylim)) ylim else c(range[1] - .025*(range[2] - range[1]),                                              
+                             range[2] + .025*(range[2] - range[1]))
+            tickmarks <- if (type == "response") make.ticks(ylim, 
+                         link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
+                         else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
             nm <- names(x)[1]
             x.vals <- x.data[, nm]   
             if (nm %in% names(ticks.x)){
@@ -244,11 +261,21 @@ plot.eff <- function(x, x.var=which.max(levels),
                 strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
                 panel=function(x, y, x.vals, rug, lower, upper, has.se, ...){
                     if (grid) panel.grid()
-                    llines(x, y, lwd=2, col=colors[1], ...)
+                    good <- !is.na(y)
+                    axis.length <- diff(range(x))
+                    llines(x[good], y[good], lwd=2, col=colors[1], ...)
                     if (rug) lrug(x.vals)
-                    if (has.se){
-                        llines(x, lower, lty=2, col=colors[2])
-                        llines(x, upper, lty=2, col=colors[2])
+                    if (has.se){  
+                        if (ci.style == "bars"){
+                            larrows(x0=x[good], y0=lower[good], 
+                                    x1=x[good], y1=upper[good], 
+                                    angle=90, code=3, col=eval(colors[2]), 
+                                    length=.125*cex/1.5)
+                        }
+                        else{ if(ci.style == "lines") {
+                            llines(x[good], lower[good], lty=2, col=colors[2])
+                            llines(x[good], upper[good], lty=2, col=colors[2])
+                        }}
                     }
                     if (has.thresholds){
                         panel.abline(h=thresholds, lty=3)
@@ -275,6 +302,7 @@ plot.eff <- function(x, x.var=which.max(levels),
         }
         return(result)
     }
+###  more than one variate
     predictors <- names(x)[1:n.predictors]
     levels <- sapply(apply(x[,predictors], 2, unique), length)
     if (is.character(x.var)) {
@@ -288,15 +316,24 @@ plot.eff <- function(x, x.var=which.max(levels),
         z.var <- which.z
     }    
     if (x.var == z.var) z.var <- z.var + 1
-    range <- if (has.se && (!multiline)) range(c(x$lower, x$upper)) else range(x$fit)
-    ylim <- if (!missing(ylim)) ylim else c(range[1] - .025*(range[2] - range[1]),                                              
-        range[2] + .025*(range[2] - range[1]))
-    tickmarks <- if (type == "response") make.ticks(ylim, link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
-    else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
-    if (multiline){
+### multiline
+    if (multiline){ 
+        ci.style <- if(is.null(ci.style)) "none" else ci.style
+        if(ci.style == "lines") { 
+            cat("Confidence interval style 'lines' changed to 'bars'\n")
+            ci.style <- "bars"}
+        range <- if (has.se && ci.style !="none")
+                  range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
+        ylim <- if (!missing(ylim)) ylim else c(range[1] - .025*(range[2] - range[1]),                                              
+                    range[2] + .025*(range[2] - range[1]))
+        tickmarks <- if (type == "response") make.ticks(ylim, link=trans.link, 
+             inverse=trans.inverse, at=ticks$at, n=ticks$n)
+             else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
         zvals <- unique(x[, z.var])
         if (length(zvals) > min(c(length(colors), length(lines), length(symbols))))
             stop(paste('Not enough colors, lines, or symbols to plot', length(zvals), 'lines'))
+        
+### multiline factor
         if (is.factor(x[,x.var])){
             levs <- levels(x[,x.var])
             key<-list(title=predictors[z.var], cex.title=1, border=TRUE,
@@ -309,13 +346,23 @@ plot.eff <- function(x, x.var=which.max(levels),
                     if (n.predictors > 2) paste(" |", 
                         paste(predictors[-c(x.var, z.var)])), collapse="*"))),
                 strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
-                panel=function(x, y, subscripts, z, ...){
+                panel=function(x, y, subscripts, z, lower, upper, show.se, ...){
                     if (grid) panel.grid()
                     for (i in 1:length(zvals)){
                         sub <- z[subscripts] == zvals[i]
-                        llines(x[sub], y[sub], lwd=2, type='b', col=colors[i], 
+                        good <- !is.na(y[sub])
+                        os <- if(show.se)
+                              (i - (length(zvals) + 1)/2) * (2/(length(zvals)-1)) * 
+                                    .01 * (length(zvals) - 1) else 0
+                        llines(x[sub][good]+os, y[sub][good], lwd=2, type='b', col=colors[i], 
                             pch=symbols[i], lty=lines[i], cex=cex, ...)
-                    }
+                        if (show.se){
+                            larrows(x0=x[sub][good]+os, y0=lower[subscripts][sub][good], 
+                                    x1=x[sub][good]+os, y1=upper[subscripts][sub][good], 
+                                    angle=90, code=3, col=eval(colors[i]), 
+                                    length=.125*cex/1.5)
+                        }
+                    } 
                     if (has.thresholds){
                         panel.abline(h=thresholds, lty=3)
                         panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)), 
@@ -323,7 +370,7 @@ plot.eff <- function(x, x.var=which.max(levels),
                         panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)), 
                             thresholds, threshold.labels, adj=c(1,0), cex=0.75)
                     }
-                },
+                },        
                 ylim=ylim,
                 ylab=ylab,
                 xlab=if (missing(xlab)) predictors[x.var] else xlab,
@@ -334,13 +381,16 @@ plot.eff <- function(x, x.var=which.max(levels),
                 zvals=zvals,
                 main=main,
                 key=key,
+                lower=x$lower, upper=x$upper, 
+                show.se=has.se && ci.style=="bars", 
                 data=x, ...)
             result <- update(plot, layout = if (missing(layout)) 
                 c(0, prod(dim(plot))) else layout)
             result$split <- split
             result$more <- more
             class(result) <- c("plot.eff", class(result))
-        }    
+        } 
+### multiline variate   
         else{
             nm <- names(x)[x.var]
             x.vals <- x.data[, nm]   
@@ -373,12 +423,25 @@ plot.eff <- function(x, x.var=which.max(levels),
                     if (n.predictors > 2) paste(" |", 
                         paste(predictors[-c(x.var, z.var)])), collapse="*"))),
                 strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
-                panel=function(x, y, subscripts, x.vals, rug, z, ...){
+# old           panel=function(x, y, subscripts, x.vals, rug, z, ...){
+                panel=function(x, y, subscripts, x.vals, rug, z, lower, upper, show.se, ...){
                     if (grid) panel.grid()
                     if (rug) lrug(x.vals)
+#new
+                    axis.length <- diff(range(x))
+#end 
                     for (i in 1:length(zvals)){
                         sub <- z[subscripts] == zvals[i]
-                        llines(x[sub], y[sub], lwd=2, type='l', col=colors[i], lty=lines[i], cex=cex, ...)
+                        good <- !is.na(y[sub])
+                        llines(x[sub][good], y[sub][good], lwd=2, type='l', col=colors[i], lty=lines[i], cex=cex, ...)
+                        if(show.se){
+                           os <- (i - (length(zvals) + 1)/2) * (2/(length(zvals)-1)) * 
+                                    .01 * axis.length
+                           larrows(x0=x[sub][good]+os, y0=lower[subscripts][sub][good], 
+                                    x1=x[sub][good]+os, y1=upper[subscripts][sub][good], 
+                                    angle=90, code=3, col=eval(colors[i]), 
+                                    length=.125*cex/1.5)
+                        }
                     }
                     if (has.thresholds){
                         panel.abline(h=thresholds, lty=3)
@@ -397,6 +460,10 @@ plot.eff <- function(x, x.var=which.max(levels),
                 zvals=zvals,
                 main=main,
                 key=key, 
+#
+                lower=x$lower, upper=x$upper, 
+                show.se=has.se && ci.style =="bars",
+#
                 data=x, scales=list(y=list(at=tickmarks$at, labels=tickmarks$labels),
                     rot=roty, x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx), 
                     alternating=alternating),  ...)
@@ -407,20 +474,37 @@ plot.eff <- function(x, x.var=which.max(levels),
             class(result) <- c("plot.eff", class(result))
         }
         return(result)
-    }
+    } 
+# multiplot factor
+        ci.style <- if(is.null(ci.style)){
+          if(is.factor(x[, x.var])) "bars" else "lines"} else ci.style
+        range <- if (has.se && ci.style !="none")
+                  range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
+        ylim <- if (!missing(ylim)) ylim else c(range[1] - .025*(range[2] - range[1]),                                              
+                    range[2] + .025*(range[2] - range[1]))
+        tickmarks <- if (type == "response") make.ticks(ylim, link=trans.link, 
+             inverse=trans.inverse, at=ticks$at, n=ticks$n)
+             else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)  
     if (is.factor(x[,x.var])){
         levs <- levels(x[,x.var])
         plot <- xyplot(eval(parse( 
             text=paste("fit ~ as.numeric(", predictors[x.var], ") |", 
                 paste(predictors[-x.var], collapse="*")))),
             strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
-            panel=function(x, y, subscripts, lower, upper, has.se, ...){
+            panel=function(x, y, subscripts, lower, upper, has.se, ...){  
                 if (grid) panel.grid()
-                llines(x, y, lwd=2, type='b', col=colors[1], pch=19, cex=cex, ...)
+                good <- !is.na(y)
                 if (has.se){
-                    llines(x, lower[subscripts], lty=2, col=colors[2])
-                    llines(x, upper[subscripts], lty=2, col=colors[2])
+                    if (ci.style == "bars"){
+                        larrows(x0=x[good], y0=lower[subscripts][good], x1=x[good], y1=upper[subscripts][good], 
+                            angle=90, code=3, col=colors[2], length=0.125*cex/1.5)
+                    }
+                    else{ if(ci.style == "lines") {
+                        llines(x[good], lower[subscripts][good], lty=2, col=colors[2])
+                        llines(x[good], upper[subscripts][good], lty=2, col=colors[2])
+                    }}
                 }
+                llines(x[good], y[good], lwd=2, type='b', col=colors[1], pch=19, cex=cex, ...)
                 if (has.thresholds){
                     panel.abline(h=thresholds, lty=3)
                     panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)), 
@@ -441,7 +525,8 @@ plot.eff <- function(x, x.var=which.max(levels),
         result$split <- split
         result$more <- more
         class(result) <- c("plot.eff", class(result))
-    }    
+    } 
+### multiplot variate   
     else{
         nm <- names(x)[x.var]
         x.vals <- x.data[, nm]   
@@ -471,12 +556,21 @@ plot.eff <- function(x, x.var=which.max(levels),
             strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
             panel=function(x, y, subscripts, x.vals, rug, lower, upper, has.se, ...){
                 if (grid) panel.grid()
-                llines(x, y, lwd=2, col=colors[1], ...)
+                good <- !is.na(y)
+                llines(x[good], y[good], lwd=2, col=colors[1], ...)
                 if (rug) lrug(x.vals)
-                if (has.se){
-                    llines(x, lower[subscripts], lty=2, col=colors[2])
-                    llines(x, upper[subscripts], lty=2, col=colors[2])
-                }
+                if (has.se){  
+                    if (ci.style == "bars"){
+                            larrows(x0=x[good], y0=lower[subscripts][good], 
+                                    x1=x[good], y1=upper[subscripts][good], 
+                                    angle=90, code=3, col=eval(colors[2]), 
+                                    length=.125*cex/1.5)
+                        }
+                        else{ if(ci.style == "lines") {
+                            llines(x[good], lower[subscripts][good], lty=2, col=colors[2])
+                            llines(x[good], upper[subscripts][good], lty=2, col=colors[2])
+                        }}
+                    }
                 if (has.thresholds){
                     panel.abline(h=thresholds, lty=3)
                     panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)), 
@@ -626,13 +720,16 @@ plot.effpoly <- function(x,
     ylab=paste(x$response, " (", type, ")", sep=""), 
     main=paste(effect, "effect plot"),
     colors, symbols=1:10, lines=1:10, cex=1.5, 
-    factor.names=TRUE, style=c("lines", "stacked"), 
+    factor.names=TRUE, ci.style,
+    style=c("lines", "stacked"), 
     confint=(style == "lines" && !is.null(x$confidence.level)), 
     transform.x=NULL, ticks.x=NULL, xlim=NULL,
     ylim, rotx=0, alternating=TRUE, roty=0, grid=FALSE,
     layout, key.args=NULL,
     row=1, col=1, nrow=1, ncol=1, more=FALSE, ...){     
     require(lattice)
+    ci.style <- if(missing(ci.style)) NULL else 
+       match.arg(ci.style, c("bars", "lines", "none"))
     type <- match.arg(type)
     style <- match.arg(style)
     has.se <- !is.null(x$confidence.level) 
@@ -672,7 +769,7 @@ plot.effpoly <- function(x,
         if (length(which.x) == 0) stop(paste("x.var = '", x.var, "' is not in the model.", sep=""))
         x.var <- which.x
     }
-    x.vals <- x.frame[, names(x.frame)[x.var]]	
+    x.vals <- x.frame[, names(x.frame)[x.var]]    
     response <-matrix(0, nrow=nrow(x.frame), ncol=n.y.lev)
     for (i in 1:length(x$y.lev)){
         level <- which(colnames(x$prob)[i] == ylevel.names)
@@ -698,6 +795,10 @@ plot.effpoly <- function(x,
     n.predictor.cats <- sapply(data[, predictors[-c(x.var)], drop=FALSE], 
         function(x) length(unique(x)))
     if (length(n.predictor.cats) == 0) n.predictor.cats <- 1
+    ci.style <- if(is.null(ci.style)) {
+       if(is.factor(x$data[[predictors[x.var]]])) "bars" else "lines"} else ci.style
+    if( ci.style=="none" ) confint <- FALSE
+### no confidence intervals if confint == FALSE or ci.style=="none"
     if (!confint){ # plot without confidence bands
         layout <- if (missing(layout)){
             lay <- c(prod(n.predictor.cats[-(n.predictors - 1)]), 
@@ -727,7 +828,8 @@ plot.effpoly <- function(x,
                         if (grid) panel.grid()
                         for (i in 1:n.y.lev){
                             sub <- z[subscripts] == y.lev[i]
-                            llines(x[sub], y[sub], lwd=2, type="b", col=colors[i], lty=lines[i], 
+                            good <- !is.na(y[sub])
+                            llines(x[sub][good], y[sub][good], lwd=2, type="b", col=colors[i], lty=lines[i], 
                                 pch=symbols[i], cex=cex, ...)
                         }
                     },
@@ -748,7 +850,7 @@ plot.effpoly <- function(x,
                     data=data, ...)
                 result$split <- split
                 result$more <- more
-                class(result) <- c("plot.eff", class(result))			
+                class(result) <- c("plot.eff", class(result))    		
             }
             else { # x-variable numeric
                 nm <- predictors[x.var]
@@ -789,7 +891,8 @@ plot.effpoly <- function(x,
                         if (rug) lrug(x.vals)
                         for (i in 1:n.y.lev){
                             sub <- z[subscripts] == y.lev[i]
-                            llines(x[sub], y[sub], lwd=2, type="l", col=colors[i], lty=lines[i], ...)
+                            good <- !is.na(y[sub])
+                            llines(x[sub][good], y[sub][good], lwd=2, type="l", col=colors[i], lty=lines[i], ...)
                         }
                     },
                     ylab=ylab,
@@ -900,6 +1003,7 @@ plot.effpoly <- function(x,
             }
         }
     }
+### with confidence banks
     else{ # plot with confidence bands
         layout <- if(missing(layout)) c(prod(n.predictor.cats), length(levels(response)), 1) 
         else layout
@@ -911,6 +1015,7 @@ plot.effpoly <- function(x,
             lower <- lower.logit
             upper <- upper.logit
         }
+### factor
         if (is.factor(x$data[[predictors[x.var]]])){ # x-variable a factor
             levs <- levels(x$data[[predictors[x.var]]])
             result <- xyplot(eval(if (type=="probability") 
@@ -928,9 +1033,17 @@ plot.effpoly <- function(x,
                 strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
                 panel=function(x, y, subscripts, x.vals, rug, lower, upper, ... ){
                     if (grid) panel.grid()
-                    llines(x, y, lwd=2, type="b", pch=19, col=colors[1], cex=cex, ...)
-                    llines(x, lower[subscripts+as.numeric(rownames(data)[1])-1], lty=2, col=colors[2])
-                    llines(x, upper[subscripts+as.numeric(rownames(data)[1])-1], lty=2, col=colors[2])
+                    good <- !is.na(y)
+                    llines(x[good], y[good], lwd=2, type="b", pch=19, col=colors[1], cex=cex, ...)
+                    if (ci.style == "bars"){
+                        larrows(x0=x[good], y0=lower[subscripts+as.numeric(rownames(data)[1])-1][good], 
+                            x1=x[good], y1=upper[subscripts+as.numeric(rownames(data)[1])-1][good], 
+                            angle=90, code=3, col=colors[2], length=0.125*cex/1.5)
+                    }
+                    else { if(ci.style == "lines"){
+                        llines(x[good], lower[subscripts+as.numeric(rownames(data)[1])-1][good], lty=2, col=colors[2])
+                        llines(x[good], upper[subscripts+as.numeric(rownames(data)[1])-1][good], lty=2, col=colors[2])
+                    } }
                 },
                 ylab=ylab,
                 ylim= if (missing(ylim)) c(min(lower), max(upper)) else ylim,
@@ -988,9 +1101,17 @@ plot.effpoly <- function(x,
                 panel=function(x, y, subscripts, x.vals, rug, lower, upper, ... ){
                     if (grid) panel.grid()
                     if (rug) lrug(x.vals)
-                    llines(x, y, lwd=2, col=colors[1], ...)
-                    llines(x, lower[subscripts+as.numeric(rownames(data)[1])-1], lty=2, col=colors[2])
-                    llines(x, upper[subscripts+as.numeric(rownames(data)[1])-1], lty=2, col=colors[2])
+                    good <- !is.na(y)
+                    llines(x[good], y[good], lwd=2, col=colors[1], ...)
+                    if (ci.style == "bars"){
+                        larrows(x0=x[good], y0=lower[subscripts+as.numeric(rownames(data)[1])-1][good], 
+                            x1=x[good], y1=upper[subscripts+as.numeric(rownames(data)[1])-1][good], 
+                            angle=90, code=3, col=colors[2], length=0.125*cex/1.5)
+                    }
+                    else { if(ci.style == "lines"){
+                        llines(x[good], lower[subscripts+as.numeric(rownames(data)[1])-1][good], lty=2, col=colors[2])
+                        llines(x[good], upper[subscripts+as.numeric(rownames(data)[1])-1][good], lty=2, col=colors[2])
+                    } }
                 },
                 ylab=ylab,
                 xlim=trans(xlm),
